/*
	The DIY Prime Wake Up Light
	electronics.nilshugo.se
	
	Copyright 2012 Christian Larsson 
	electronics@nilshugo.se
*/

#define ADJUST_GROUP_MINUTES 0
#define ADJUST_GROUP_HOURS 1

#include "buttons.h"
#include "dispControl.h"
#include "alarmFunc.h"
#include "miscIO.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>

struct timeStruct{
	unsigned char hours;
	unsigned char minutes;
	unsigned char seconds;
};

void primeCalculation(unsigned long int* currentLargestPrime);
void initializeController(void);
void adjustTime(struct timeStruct* time);
void adjustAlarmTime(struct timeStruct *test);

volatile unsigned char digits[4];
volatile long int totalDayTimeInSeconds;
volatile long int alarmTimeInSeconds;
volatile unsigned char blinkToggleStatus = 0;
volatile unsigned char switchModeActive = 0;

ISR(TIMER0_COMP_vect){
	totalDayTimeInSeconds++;
}

ISR(TIMER2_COMP_vect){
	static unsigned char dispNr = 0;
	static unsigned char blinkCounter = 0;
	static unsigned int switchModeCounter = 0;
	
	shutOffDisp();
	selectDisp(dispNr);
	printNr(digits[dispNr]);
	
	dispNr ++;
	if(dispNr > 3)
		dispNr = 0;

	if(blinkToggleStatus){
		if(blinkCounter  > 150){
			blinkToggleStatus = BLINK_OFF;
			blinkCounter  = 0;	
		}		
	}
	else{
		if(blinkCounter  > 30){
			blinkToggleStatus = BLINK_ON;
			blinkCounter  = 0;	
		}	
	}
	
	if(switchModeCounter > 500){
		switchModeActive++;
		switchModeCounter = 0;		
	}

	blinkCounter++;
	switchModeCounter++;
}

int main(void){
	struct timeStruct time, alarmTime;
	unsigned char groupAdjusted;
	unsigned long int currentLargestPrime;
	
	initializeController();

	totalDayTimeInSeconds	= 0;
	alarmTimeInSeconds		= 0;
	
	currentLargestPrime		= 0; //largest possible prime 4294967296
	
	setAlarmStatus(ALARM_OFF);

	controlDot(0, 1, 0, 0);

	sei();
	
    while(1){
		adjustTime(&time);
		
		if(switchIsActiveLight()){
			setNewDimmingSettingSlow(DIM_MAX_VALUE);
		}
		else{
			if(switchIsActiveAlarm())
				setAlarmStatus(ALARM_ON);
			else
				setAlarmStatus(ALARM_OFF);
		
			if(isAlarmActive()){
				controlAlarm(totalDayTimeInSeconds, alarmTimeInSeconds);
			}			
		}

		switch(switchModeStatus()){
			case MODE_VIEW_TIME:
				if(switchModeActive == 0){
					controlDot(0, 0, 0, 0);
					controlDisplayContentThreeDigits(digits, (currentLargestPrime/1000000));
				}
				else if(switchModeActive == 1){
					controlDot(0, 0, 0, 0);
					controlDisplayContentThreeDigits(digits, (currentLargestPrime/1000));
				}
				else if(switchModeActive == 2){
					controlDot(0, 0, 0, 0);
					controlDisplayContentThreeDigits(digits, (currentLargestPrime));
				}
				else{
					if(switchModeActive > 5)
						switchModeActive = 0;

					if(buttonIsPressedChange())
						controlDisplayContentTime(BLINK_NONE, blinkToggleStatus, digits, time.minutes, time.seconds);
					else
						controlDisplayContentTime(BLINK_NONE, blinkToggleStatus, digits, time.hours, time.minutes);				
				}	
				break;
			
			case MODE_SET_ALARM:
				groupAdjusted = ADJUST_GROUP_MINUTES;
				
				if(!switchIsActiveLight())
					setAlarmStatus(ALARM_OFF);
									
				while(switchModeStatus() == MODE_SET_ALARM){
					adjustAlarmTime(&alarmTime);
					
					if(buttonIsPressedChange()){
						_delay_ms(BUTTON_DELAY_TIME);
						
						if(groupAdjusted == ADJUST_GROUP_MINUTES)
							groupAdjusted = ADJUST_GROUP_HOURS;
						else
							groupAdjusted = ADJUST_GROUP_MINUTES;
					}
					
					if(groupAdjusted == ADJUST_GROUP_MINUTES){
						if(buttonIsPressedPlus()){
							_delay_ms(BUTTON_DELAY_TIME);
							alarmTimeInSeconds += SECONDS_IN_MINUTE;
							alarmTimeInSeconds -= alarmTimeInSeconds % SECONDS_IN_MINUTE;
						}
						else if(buttonIsPressedMinus()){
							_delay_ms(BUTTON_DELAY_TIME);
							alarmTimeInSeconds -= SECONDS_IN_MINUTE;
							alarmTimeInSeconds -= alarmTimeInSeconds % SECONDS_IN_MINUTE;
						}

						controlDisplayContentTime(BLINK_SECOND, blinkToggleStatus, digits, alarmTime.hours, alarmTime.minutes);
					}
					else{
						if(buttonIsPressedPlus()){
							_delay_ms(BUTTON_DELAY_TIME);
							alarmTimeInSeconds += SECONDS_IN_HOUR;
							alarmTimeInSeconds -= alarmTimeInSeconds % SECONDS_IN_MINUTE;
						}
						else if(buttonIsPressedMinus()){
							_delay_ms(BUTTON_DELAY_TIME);
							alarmTimeInSeconds -= SECONDS_IN_HOUR;
							alarmTimeInSeconds -= alarmTimeInSeconds % SECONDS_IN_MINUTE;
						}
						controlDisplayContentTime(BLINK_FIRST, blinkToggleStatus, digits, alarmTime.hours, alarmTime.minutes);
					}
				}				
				break;

			case MODE_SET_TIME:
				groupAdjusted = ADJUST_GROUP_MINUTES;
				
				if(!switchIsActiveLight())
					setAlarmStatus(ALARM_OFF);
				
				while(switchModeStatus() == MODE_SET_TIME){
					adjustTime(&time);
					
					if(buttonIsPressedChange()){
						_delay_ms(BUTTON_DELAY_TIME);
						
						if(groupAdjusted == ADJUST_GROUP_MINUTES)
							groupAdjusted = ADJUST_GROUP_HOURS;
						else
							groupAdjusted = ADJUST_GROUP_MINUTES;
					}
					
					if(groupAdjusted == ADJUST_GROUP_MINUTES){
						if(buttonIsPressedPlus()){
							_delay_ms(BUTTON_DELAY_TIME);
							totalDayTimeInSeconds += SECONDS_IN_MINUTE;
							totalDayTimeInSeconds -= totalDayTimeInSeconds % SECONDS_IN_MINUTE;
						}
						else if(buttonIsPressedMinus()){
							_delay_ms(BUTTON_DELAY_TIME);
							totalDayTimeInSeconds -= SECONDS_IN_MINUTE;
							totalDayTimeInSeconds -= totalDayTimeInSeconds % SECONDS_IN_MINUTE;
						}

						controlDisplayContentTime(BLINK_SECOND, blinkToggleStatus, digits, time.hours, time.minutes);
					}
					else{
						if(buttonIsPressedPlus()){
							_delay_ms(BUTTON_DELAY_TIME);
							totalDayTimeInSeconds += SECONDS_IN_HOUR;
							totalDayTimeInSeconds -= totalDayTimeInSeconds % SECONDS_IN_MINUTE;
						}
						else if(buttonIsPressedMinus()){
							_delay_ms(BUTTON_DELAY_TIME);
							totalDayTimeInSeconds -= SECONDS_IN_HOUR;
							totalDayTimeInSeconds -= totalDayTimeInSeconds % SECONDS_IN_MINUTE;
						}						

						controlDisplayContentTime(BLINK_FIRST, blinkToggleStatus, digits, time.hours, time.minutes);
					}
				}
				break;	
		}
		
		primeCalculation(&currentLargestPrime);
    }
}

void primeCalculation(unsigned long int* currentLargestPrime){
	static unsigned long int currentNumberTested = 2;
	static unsigned long int currentDivisorToTest = 1;
	static unsigned long int divisorTestLimit = 1;

	if(currentDivisorToTest >= divisorTestLimit){
		*currentLargestPrime = currentNumberTested;
		currentNumberTested++;
		divisorTestLimit = sqrt(currentNumberTested);
		currentDivisorToTest = 2;
	}
	else{
		if(currentNumberTested % currentDivisorToTest == 0){ //currentNumberTested not a prime
			currentNumberTested++;
			divisorTestLimit = sqrt(currentNumberTested);
			currentDivisorToTest = 2;
		}				
		else{
			currentDivisorToTest++;
		}				
	}
		
	return;	
}

void initializeController(void){
	DISP_PORT_DDR		= 0xFF;
	DISP_PORT			= 0xFF;

	DISP_AND_RELAY_CTRL_PORT_DDR	|= 0b11111111;
	DISP_AND_RELAY_CTRL_PORT		|= 0b00001111;
	DISP_AND_RELAY_CTRL_PORT		&= 0b01111111; //Open relay

	BUTTON_AND_ALARM_STATUS_PORT_DDR		= 0b10000000;
	BUTTON_AND_ALARM_STATUS_PORT			|= 0b11111111; //Activate pull-up resistors for switches

	//Initialize timer0 which counts the mains frequency and updates current time 
	TCCR0 |= (1 << WGM01) | (1 << CS02) | (1 << CS01) | (1 << CS00);
	OCR0 = 49;
	TIMSK |= (1 << OCIE0);
	
	//Initialize timer1 which is used to control the dimming of the light
	TCCR1A |= (1 << COM1A1) | (1 << WGM11);
	TCCR1B |= (1 << CS10) | (1 << WGM12) | (1 << WGM13);
	ICR1	= 100;
	OCR1A	= 0;

	//Initialize timer2 which is mainly used for controlling the display (selecting and printing digits)
	TCCR2 |= (1 << WGM21) | (1 << CS22) | (1 << CS20);
	OCR2 = 35;
	TIMSK |= (1 << OCIE2);

	return;	
}

void adjustTime(struct timeStruct* time){
	if(totalDayTimeInSeconds >= SECONDS_IN_DAY)
		totalDayTimeInSeconds -= SECONDS_IN_DAY;
	if(totalDayTimeInSeconds < 0)
		totalDayTimeInSeconds += SECONDS_IN_DAY;

	time->hours = totalDayTimeInSeconds / SECONDS_IN_HOUR;
	time->minutes = (totalDayTimeInSeconds % SECONDS_IN_HOUR) / SECONDS_IN_MINUTE;
	time->seconds = totalDayTimeInSeconds%SECONDS_IN_MINUTE;

	return;
}

void adjustAlarmTime(struct timeStruct* alarmTime){
	if(alarmTimeInSeconds >= SECONDS_IN_DAY)
		alarmTimeInSeconds -= SECONDS_IN_DAY;
	if(alarmTimeInSeconds < 0)
		alarmTimeInSeconds += SECONDS_IN_DAY;
		
	alarmTime->hours = alarmTimeInSeconds / SECONDS_IN_HOUR;
	alarmTime->minutes = (alarmTimeInSeconds % SECONDS_IN_HOUR) / SECONDS_IN_MINUTE;
	alarmTime->seconds = 0;
}